/****************************************************************************
	TORNADO Software Development Kit (TSDK). Demo Utilities.
	DSP->Host Interrupt Demo (different IRQ Sources scan) for TORNADO-6x Boards. v.1.0
	(C) MicroLAB Systems 2000-2001. All rights reserved.

   Description:
   ------------
   This is the TSDK host PC utility, which provides sample code for handling
   host IRQ from different sources.

   This demo must run in conjunction with DSP code and perform load of 'T6D_HIG.OUT'
   during execution.

   The following host IRQ sources are tested:
	1. HM_RQ
	2. HPIC-HINT


  Notes:
   ------
	1. Host IRQ line must be set at TORNADO ISA board via on-board configuration jumper 
	   and correspondingly entered via TSDK TCFG.EXE utility. Recommended host IRQ lines 
	   are #5, #10, #12, one of which is typically not used in host PC. Use PC BIOS 'PCI features'
	   menu in order to reserve IRQ line for ISA-bus slots of your PC.
	2. TORNADO-6x on-board 'DSP RESET CONTROL' jumper must be set to the 'HOST' position.

	3. Usage:
			T6_INT [-iN],
		where N is the board index if multiple TORNADO boards are installed.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com
****************************************************************************/

#include <wtypes.h>
#include <stdio.h>
#include <conio.h>

#include "th.h"	// definitions

extern void  t6x_irq_test();

	// temporary SMP segment used if SMP is OFF for the board
#define	TEMP_SMP_SEGM	0xd8000L

int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	BOOLEAN found = FALSE, temp_smp_segm_on=FALSE;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];


	printf("\nTSDK demo: DSP->Host Interrupt Processing Demo for TORNADO-6X Boards. v.1.0\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2002\n\n");

	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    T6_INT [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_err:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;


	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	// check for TSDK API error flag
	if(err=TH_err_flag())
		goto exit_err;


	if(!TH_T6X_BOARD)
	{
		printf("This test runs on TORNADO-6x boards only.\n");
		return 0;

	}

	// check for INT line is has been asigned via TCFG utility for this ISA board
	if(TH_interrupt_line()==0)
	{
		printf("No interrupt line has been assigned. Use TCFG.EXE to assign INTERRUPT line.\n");
		return 0;

	}
	

	// temporary SMP segment used if SMP is OFF for the board
#define	TEMP_SMP_SEGM	0xd8000L

	//check for SMP segment is set. If not set, then set it to 0xd8000, and then switch off on exit
	if(TH_smp_segm()==0)
	{
		temp_smp_segm_on=TRUE;
		TH_set_smp_segm(TEMP_SMP_SEGM);
		printf("Temporary %lx SMP segment is used.\n",TEMP_SMP_SEGM);
	}

	t6x_irq_test();

	// check whether SMP was temporary set to ON in this DEMO
	if(temp_smp_segm_on)
		TH_set_smp_off();

	return 1;

}
