/**********************************************************************************
   DSP-to-Host IRQ Generator for TORNADO-P64XX boards.
   (C) MicroLAB Systems, 2000-2004

   file: C64xx DSP kernel

   Description:
   ------------
   This is DSP resident utility for TORNADO board, which must be loaded and 
   initialized from host PC interrupt demo application. This DSP code
   generates DSP-to-Host IRQ sources as the following:
	1. If DPRAM_HM_RQ is detected, then DSP reads DPRAM_HM_RQ data and writes
	   (inverted+1) data to the DPRAM_MH_RQ in order to generate the DPRAM_IRQ to host.
	   DPRAM_HM_RQ is supported for TP64xx boards with 256K/512Kx32 DPRAM and is not
	   supported for TP64xx boards with 128Kx32 DPRAM.
	2. If HM_RQ0 is detected, then DSP reads HM_RQ0 data and generates the MH_RQ to host.
	3. If HPIC-DSPINT is detected, then DSP clears DSPINT and sets the HPIC-HINT to host.
	4. If IMBX-0..2 is coming FULL, then DSP reads IMBX#n data and writes
	   (inverted) data to OMBX-0..2 correspondingly in order to generate 
	   OMBX EMPTY and IMBX FULL host interrupt requests.

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: support@mlabsys.com

***********************************************************************************/

#define	__TP6X_SELECT_TP64XX__		1
#define	__TP6X_DPRAM256K__			1
#include "c6x_def.h"
#include "tp6x_dsp.h"


#define	MCD_ADDR				(TP64XX_DPRAM_SADDR+0x200)	// MCD address for TSDK tests

#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0

#define	MSTAT_SYNC				3
#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-2
#define	ERR_INV_PARAM			-1
#define	ERR_CMND_EXEC_ERR		-3
#define	ERR_INV_INIT			-100


struct MCD_DD
{ 
	int  		m_id,
				prg_id,
				cmnd,
				status,
				error;

	unsigned    up[4];
	unsigned    dpram_len;
};

struct MCD_DD *MCD;

unsigned		*dpram_hm_rq, 
				*dpram_mh_rq,
				dpram_irq_support=TP6X_OFF;



void init_irq()
{
	int i;

	// configure and disable all IRQs
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_POS);
	C6X_INT_DISABLE_ALL;


	// disable all IRQs
	C6X_INT_DISABLE_ALL;

	// clear all MBX/FIFO flags
    TP6X_set_pcic_agcsts(TP6X_PCIC_AGCSTS_CLR_ALL_MBX_FIFO_STAT_FLAGS);	

	// clear DPRAM_HM_IRQ if any and if it is enabled
	if(dpram_irq_support==TP6X_ON)
		i= *dpram_hm_rq;	

	// clear HM_RQ0 is any
	i= TP6X_hm_rq0();
	TP6X_clr_hm_rq0();

	// clear HPI-DSPINT if any
	TP6X_clr_dspint();

	// configure DSP on-chip interrupt multipxer 
	// INT4 and INT5 are used as default mapped to external XINT4 and XINT5 interrupt request inputs
	C6X_SET_INTMUX(4,C6X_INTSEL_XINT4);
	C6X_SET_INTMUX(5,C6X_INTSEL_XINT5);

	// now configure external IRQ sources in order to trace DPRAM_HM_RQ and HM_RQ0
	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_HM_RQ0);	// INT4=HM_RQ0

	// enable MDPIRQ if it is supported
	if(dpram_irq_support==TP6X_ON)
		TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_MDPIRQ);	// INT5=MDPIRQ
	else
		TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_OFF);	// disable INT5 selector

	// clear all IRQs
	C6X_INT_CLEAR_ALL;

}




main()
{
	// init C64xx DSP EMIF
	TP6X_init_dsp_emif();

	// initialize communication descriptor
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x98765432;
	MCD->up[1]=0xabcdef01;

	// now check whether DPRAM IRQs are supported via the MCD->dpram_len parameter
	if(MCD->dpram_len>0x80000)
	{	
		// this is 256Kx32 DPRAM and higher, DPRAM IRQs are supported. Assign DPRAM IRQ pointers.
		dpram_irq_support=TP6X_ON;
		dpram_hm_rq = (unsigned *)(unsigned)(TP64XX_DPRAM_SADDR+ MCD->dpram_len -4L);
		dpram_mh_rq = (unsigned *)(unsigned)(TP64XX_DPRAM_SADDR+ MCD->dpram_len -8L);
	}

	// init IRQs in order to trace external DPRAM_HM_RQ and HM_RQ0
	init_irq();

	// set READY flag to host CPU	
	MCD->status=MSTAT_RDY;


	// perform infinite loop for polling the input requests
	while(1)
    {
	   	// check for the DPRAM_HM_RQ via INT5 in case DPRAM supports H->M/M->H IRQs
		if(dpram_irq_support==TP6X_ON && C6X_INT_IS_ON(5))
		{
			// 	if detected, then read DPRAM_HM_RQ and 
			//	copy the (inverted+1) value to the DPRAM_MH_RQ location
			*dpram_mh_rq = (~(*dpram_hm_rq)+1);	

			// clear INT5 IRQ bit
			C6X_INT_CLEAR(5) ;
		}

	   	// check for the HM_RQ0 via INT4
		if(C6X_INT_IS_ON(4))
		{
			// 	if detected, then clear HM_RQ0 and set MH_RQ
			TP6X_clr_hm_rq0();
			TP6X_set_mh_rq();	

			// clear INT4 IRQ bit
			C6X_INT_CLEAR(4) ;
		}

		
		// now check for the HPI-DSPINT
		if(TP6X_dspint())
		{	
			// if found, then clear DSPINT and set HINT
			TP6X_clr_dspint();
			TP6X_set_hint();
		}


		// Check IMBX#0 is FULL. If FULL, then read IMBX#0 and copy inverted
		// value to the OMBX#0.
		if(TP6X_pcic_ambef() & TP6X_PCIC_AMBEF_IMBX0_BYTE3_STAT_BITMASK)
			TP6X_set_pcic_ombx(0,~TP6X_pcic_imbx(0));

		// Check IMBX#1 is FULL. If FULL, then read IMBX#1 and copy inverted
		// value to the OMBX#1.
		if(TP6X_pcic_ambef() & TP6X_PCIC_AMBEF_IMBX1_BYTE3_STAT_BITMASK)
			TP6X_set_pcic_ombx(1,~TP6X_pcic_imbx(1));

		// Check IMBX#2 is FULL. If FULL, then read IMBX#2 and copy inverted
		// value to the OMBX#2.
		if(TP6X_pcic_ambef() & TP6X_PCIC_AMBEF_IMBX2_BYTE3_STAT_BITMASK)
			TP6X_set_pcic_ombx(2,~TP6X_pcic_imbx(2));
	}
}




