/****************************************************************************
	TORNADO Software Development Kit (TSDK). Demo Utilities.
	DSP->Host Interrupt Demo (different IRQ Sources scan) for TORNADO-P6x/P64xx Boards. v.1.1
	(C) MicroLAB Systems 2000-2004. All rights reserved.

   Description:
   ------------
   This is the TSDK host PC utility, which provides sample code for handling
   host IRQ from different sources.

   This demo must run in conjunction with the DSP code and performs load of the corresponding
   DSP kernel (upon the TORNADO board type) during execution.

   The following host IRQ sources are tested:
	1. DPRAM_HM_RQ (except for the TP64xx boards with 128Kx32 DPRAM)
	2. HM_RQ0
	3. TP62/TP67/TP64xx only: HPIC-HINT
	4. PCIC IMBX-0..2

   Copyright:
   ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is assumed
	for any usage or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: support@mlabsys.com
****************************************************************************/

#include <wtypes.h>
#include <winbase.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
	

// include definitions
#include "th.h"	
#include "..\..\_util\stuffs.h" // some auxiliary functions


void  tp6x_tp64xx_irq_test();


int main(int argc, char* argv[])
{
	TH_BRD_INFO_TD board_info[TH_MAX_BOARDS];
	BOOLEAN found = FALSE;
	int err;
	ULONG num_brd,selected_brd = -1;
	CHAR  brd_name[100];


	printf("\nTSDK demo: DSP->Host Interrupt Processing Demo for TORNADO-P6x/P64xx Boards. v.1.1\n");
	printf("Copyright (C) MicroLAB Systems Ltd, 2000-2004\n\n");


	// get the total number of TORNADO boards we have
	num_brd = TH_boards_total();
	if(!num_brd)
	{
		printf("No TORNADO board(s) have been found\n");
		return 0;
	}

	printf("Total TORNADO boards detected = %u\n",num_brd);

	TH_boards_info(board_info,&num_brd);

	if(num_brd == 1)
	{
		// only one TORNADO board has been detected, then select it as default
		selected_brd = board_info[0].index;
	}
	else
		// for multiple TORNADO boards, select one that is ispecified via -iN option
		if(		argc>1 
			&&	!strncmp(argv[1],"-i",2)
			&&	!sscanf(argv[1] + 2,"%u",&selected_brd)
		  )
		{
err_usage:
			printf("Usage:\n");
			printf("    TP6x_TP64xx_INT [-iN]\n");
			printf("	where N specifies the board index if multiple TORNADO boards are installed\n");
			return 0;
		}


	// print board info
	for(ULONG brd = 0;brd<num_brd;brd++)
	{
		err = TH_board_name(board_info[brd].index,brd_name);
		if(err)
		{
exit_error:
			if(err!=TH_OK)
				printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
					
			return 0;
		}

		printf("index: #%u    board: %s",board_info[brd].index,brd_name);

		if(board_info[brd].index == selected_brd)
		{
			printf(" (selected)");
			found = 1;
		}

		printf("\n");
	}

	// check for board has been found successfully
	if(!found)
		goto err_usage;

	// select & open the board
	TH_board_select(selected_brd);
	TH_board_open();

	// check for TSDK API error flag
	if(err=TH_err_flag())
		goto exit_error;

	// verify the board type
	if(!TH_TP6X_TP64XX_BOARD)
	{
		printf("This test runs on TORNADO-P6x/P64xx boards only.\n\n");
		return 0;
	}


	// check whether the DPRAM is installed for TP64xx boards, since this test
	// requires that the DPRAM is installed
	if(TH_TP64XX_BOARD && TH_board_dpram_len()==0)
	{
		printf("This test requires that DPRAM is installed for TORNADO-P64xx boards.\n\n");
		return 0;
	}


	// run the INT test
	tp6x_tp64xx_irq_test();
	return 1;
}



//------------------ Interrupt test function ----------------------------


char err_mess[200];

ULONG *D,data,data_start, data_rd;

#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");

#define	MCD_ADDR			(0x200L)

#define	M_ID				0x110L
#define	PRG_ID				0x111L

#define	MCMND_NOP			0

#define	MSTAT_RDY			0

#define	ERR_OK				0
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1

typedef struct 
{ 
	long  	m_id,
			prg_id,
			cmnd,
			status,
			error;

   ULONG    up[4];
   ULONG    dpram_len;
} MCD_DD;

MCD_DD *MCD;


char last_err_mess[100];

#define	_PCIC_CLR_ALL_MBX_STAT_FLAGS	TH_set_pcic_mcsr_rg(TH_PCIC_MCSR_CLR_MBX_STAT_FLAGS)
#define	_PCIC_CLR_ALL_IRQ			   	TH_pcic_int_clr_all_irq()



// TP6x/TP64xx interrupt test function
void  tp6x_tp64xx_irq_test()
{
	int err;

	*err_mess=0;

	// clear control register in order to reset the DSP and disable PCI-bus mastering
	TH_set_ctrl_rg(0);
	// TP64xx board: clear the CONTROL-2 register in order to select NO BOOT mode
	if(TH_TP64XX_BOARD)
		TH_set_ctrl2_rg(0);

	// check for the API error
	if(err=TH_err_flag())
		goto exit_error;

	if(TH_TP6X_BOARD)
	{
		// TP6x boards: 
		//	- save SMP register configuration for TP6x in order to restore it on exit
		//	- set dual-page mode for access to DPRAM and DPSEM simultaneously
		TH_set_smp_mode_spa();
		TH_clr_dpram_err();
	}

	// clear HPI error if any
	TH_clr_hpi_err();

	// disable interrupts
	TH_set_im_rg(0);
	// disable interrupts via PCIC
	TH_set_pcic_intcsr_rg(0);
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if((err = TH_err_flag())) goto exit_error;

	// load DSP code, which is required to perform this test
	err=TH_load_coff(TH_TP6X_BOARD ? "dsp_src\\tp6x_hig.out":"dsp_src\\tp64xx_hig.out");

	// check for TSDK API error
	if(err!=TH_OK) goto exit_error;


	// get pointer to the MCD
	if(TH_TP6X_BOARD)
		TH_set_smp_ptr(MCD_ADDR,TH_TP6X_SMP_DPRAM_AREA,(void**)&MCD,TH_SMP_SMP0_SEL);
	else
		TH_dpram_ptr(MCD_ADDR,(void**)&MCD);


	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// initialize DSP and test correct operation
	printf("testing DSP initialization ... ");

	MCD->m_id=-1L;
	MCD->prg_id=-1L;
	MCD->cmnd=-1L;
	MCD->status=-1L;
	MCD->error=-1L;
	MCD->up[0]=0xffffffffL;
	MCD->up[1]=0xffffffffL;

	
	// pass the DPRAM length in order the DSP code could calculate the address 
	// for M->H/H->M IRQ DPRAM locations and the TP64xx board could define
	// whether the DPRAM M->H/H->M request feature is available
	MCD->dpram_len=TH_dpram_len();

	if(TH_TP6X_BOARD && TH_dpram_err_data())
	{
	    strcpy(err_mess,"DPRAM error on init comm");
	    goto exit_error;
	
	}

	// release the DSP reset
	TH_set_mgo();

	// configure the HPI port
	// Note, that we can configure the DSP on-chip HPI port only after the DSP reset has been released.
	TH_hpi_init(TH_HPI_INIT_FULL);
	
	// wait for DSP will init or timeout
	init_timer(1);  
	while (MCD->status!=MSTAT_RDY) 
    {
		if(timer_expired())
		{ 
		   strcpy(err_mess,"no reply from TP6X/TP64xx board");
	       goto exit_error;
		}

		if(TH_TP6X_BOARD && TH_dpram_err_data())
		{ 
dpram_error:
			sprintf(err_mess, "error: DPRAM error");
			goto exit_error;
		}
    }

	// validate returned MCD data
	if(		MCD->m_id!=M_ID      
	   ||   MCD->prg_id!=PRG_ID  
	   ||   MCD->cmnd!=MCMND_NOP      
	   ||   MCD->status!=MSTAT_RDY    
	   ||   MCD->error !=ERR_OK       
	   ||   MCD->up[0] !=0x98765432L      
	   ||   MCD->up[1] !=0xabcdef01L
      )
	{ 
		 strcpy(err_mess,"invalid DSP initialization");
	     goto exit_error;
	}


	// check for DPRAM access error
	if(TH_TP6X_BOARD && TH_dpram_err_data()!=0)	goto dpram_error;

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

    printf("passed\n");


	// now proceed with host IRQ test
#define START_DATA		0x24398657

	TH_INTREGS_TD regs;
	ULONG	data,mb_num;

	// clear PCIC MBX flags and all possible IRQs
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   

	// disable interrupts 
	TH_pcic_int_set_all_ie_off();

	// clear HPIC HINT for TP62/TP67/TP64xx
	if(TH_TP62_TP67_BOARD || TH_TP64XX_BOARD) 
		TH_clr_hpic_hint();		

	// clear DSP-to-Host request
	TH_clr_mh_rq();		

	// read/clear DSP-to-Host IRQ via DPRAM
	// Do not perform this for TP64xx board with 128Kx32 DPRAM, whcih does not support H->M/M->H IRQs
	if(TH_TP6X_BOARD || (TH_TP64XX_BOARD && TH_board_dpram_len()>=TP64XX_DPRAM_LEN_256KX32))
		data = TH_dpram_mh_rq_data();

	// connect TSDK host INT line
	if((err = TH_connect_interrupt())) goto exit_error;

	// clear host INT queue
	TH_clear_interrupt();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// first try with disable interrupts and check for the TIMEOUT occurs
	printf("testing with disabled interrupts...");

	data = START_DATA;
	// set OMBX#0
	TH_set_pcic_ombx_rg(0,data);

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			printf("OK. Timed out.\n");
			break;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			// this is error
			err=TH_OK;
			printf("failed.\n");
			strcpy(err_mess,"We've got unexpected interrupt at step #1");
			goto exit_error;

		default:	// this is error
			goto exit_error;
	}

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// now test host IRQs via IMBX#0
	// (DSP is expected to return inverted data fm OMBX#0 to IMBX#0)
	printf("testing DSP-to-Host IRQ via IMBX#0...");
   
	// enable host IMBX#0 interrupts on BYTE#3
	TH_pcic_int_set_imbx_ie_on(0,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(0,data);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_IMBX_FULL)
				{
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_IMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT);

					if(mb_num!= 0)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from IMBX #%lx\n",mb_num);
						goto exit_error;
					}

					if(regs.u.pci.pcic_imbx != ~data)
					{
						printf("failed\n");
						sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.pcic_imbx,  ~data);
						goto exit_error;
					}


				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// Test host IRQs via the OMBX#0 empty state.
	// This is actually the previous test, however host IRQ mask is OMBX#0 empty state.
	// (DSP is expected to return inverted data fm OMBX#0 to IMBX#0)
	printf("testing DSP-to-Host IRQ via OMBX#0 emption...");
   
	// enable host OMBX#0 BYTE#3 interrupts
	TH_pcic_int_set_ombx_ie_on(0,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(0,data);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_OMBX_EMPTY)
				{
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_OMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_OMBX_SEL_BITSHIFT);

					if(mb_num!= 0)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from OMBX #%lx emption...\n",mb_num);
						goto exit_error;
					}

				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// now test host IRQs via IMBX#1
	// (DSP is expected to return inverted data fm OMBX#1 to IMBX#1)
	printf("testing DSP-to-Host IRQ via IMBX#1...");
   
	// enable host IMBX#1 interrupts on BYTE#3
	TH_pcic_int_set_imbx_ie_on(1,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(1,data);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_IMBX_FULL)
				{
					err=TH_OK;
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_IMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT);

					if(mb_num!=1)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from IMBX #%lx\n",mb_num);
						goto exit_error;
					}

					if(regs.u.pci.pcic_imbx != ~data)
					{
						printf("failed\n");
						sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.pcic_imbx,  ~data);
						goto exit_error;
					}
				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}

			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;


	// now test host IRQs via IMBX#2
	// (DSP is expected to return inverted data fm OMBX#2 to IMBX#2)
	printf("testing DSP-to-Host IRQ via IMBX#2...");
   
	// enable host IMBX#2 interrupts on BYTE#3
	TH_pcic_int_set_imbx_ie_on(2,3);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_pcic_ombx_rg(2,data);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id == TH_INT_ID_PCIC_IMBX_FULL)
				{
					// check mailbox number
					mb_num = ((regs.u.pci.pcic_intcsr & TH_PCIC_INTCSR_IMBX_SEL_BITMASK)>>TH_PCIC_INTCSR_IMBX_SEL_BITSHIFT);

					if(mb_num!= 2)
					{
						printf("failed\n");
						sprintf(err_mess,"unexpected IRQ from IMBX #%lx\n",mb_num);
						goto exit_error;
					}

					if(regs.u.pci.pcic_imbx != ~data)
					{
						printf("failed\n");
						sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.pcic_imbx,  ~data);
						goto exit_error;
					}
				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}
			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}


	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;


	// now test host IRQs via DPRAM_MH_RQ
	// (DSP is expected to return inverted data fm DPRAM_HM_RQ to the DPRAM_MH_RQ)
	// Note, that this test runs with TP6x boards and TP64xx boards with 256K/512Kx32 DPRAM and
	// does not run at TP64xx boards with 128Kx32 DPRAM.
	if(TH_TP6X_BOARD || (TH_TP64XX_BOARD && TH_board_dpram_len()>=TP64XX_DPRAM_LEN_256KX32))
	{
		printf("testing DSP-to-Host IRQ via DPRAM_MH_RQ...");


		// enable host IMBX#3 interrupts on BYTE#3 in order to enable extrenal PCIC IRQs
		TH_pcic_int_set_imbx_ie_on(3,3);

		// set DPRAM_MH_RQ IE bit in HIF IM register
		TH_set_im_rg(TP6X_IM_DPRAM_IE);

		// clear possible previous INTREG queue
		TH_clear_interrupt();
   
		TH_set_dpram_hm_rq(data);

		// check for TSDK API error
		if(err=TH_err_flag()) goto exit_error;

		switch(err=TH_wait_interrupt(100))
		{
			case TH_INTERRUPT_NOT_DETECTED: // timed out
				err=TH_OK;
				printf("failed\n");
				strcpy(err_mess,"there is no reply from TORNADO via host INT");
				goto exit_error;

			case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
				if(TH_interrupt_registers(&regs))
				{
					err=TH_OK;
					// check interrupt source
					if(regs.id == TH_INT_ID_DPRAM_IRQ)
					{
						// check returned data
						if(regs.u.pci.dpram_mh_rq_data != (~data+1))
						{
							printf("failed\n");
							sprintf(err_mess,"incorrect data: returned 0x%x, expected 0x%x ",regs.u.pci.dpram_mh_rq_data,  ~data+1);
							goto exit_error;
						}
					}
					else
					{
						printf("failed\n");
						strcpy(err_mess,"unexpected host INT source.");
						goto exit_error;
					}
				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"INT register queue is empty");
					goto exit_error;
				}

				break;

			default:	// this is error
				goto exit_error;
		}


		data++;
		TH_clear_interrupt();
		printf("OK.\n");

		_PCIC_CLR_ALL_MBX_STAT_FLAGS;
		_PCIC_CLR_ALL_IRQ;   
		TH_pcic_int_set_all_ie_off();

		// check for TSDK API error
		if(err=TH_err_flag()) goto exit_error;
	}


	// now test host IRQs via MH_RQ
	// (DSP is expected to generate MH_RQ as a reply to HM_RQ0)
	printf("testing DSP-to-Host IRQ via MH_RQ...");
   

	// enable host IMBX#3 interrupts on BYTE#3 in order to enable extrenal PCIC IRQs
	TH_pcic_int_set_imbx_ie_on(3,3);

	// set DPRAM_MH_RQ IE bit in HIF IM register
	TH_set_im_rg(TP6X_IM_MH_RQ_IE);

	// clear possible previous INTREG queue
	TH_clear_interrupt();
   
	TH_set_hm_rq0_rg(1);

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	switch(err=TH_wait_interrupt(100))
	{
		case TH_INTERRUPT_NOT_DETECTED: // timed out
			err=TH_OK;
			printf("failed\n");
			strcpy(err_mess,"there is no reply from TORNADO via host INT");
			goto exit_error;

		case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
			if(TH_interrupt_registers(&regs))
			{
				err=TH_OK;
				// check interrupt source
				if(regs.id != TH_INT_ID_MH_RQ)
				{
					printf("failed\n");
					strcpy(err_mess,"unexpected host INT source.");
					goto exit_error;
				}
			}
			else
			{
				printf("failed\n");
				strcpy(err_mess,"INT register queue is empty");
				goto exit_error;
			}

			break;

		default:	// this is error
			goto exit_error;
	}

	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;


	// now test host IRQs via HPIC HINT for TP62/TP67/TP64xx boards only
	// (DSP is expected to generate MH_RQ as a reply to HM_RQ0)
	if(TH_TP62_TP67_BOARD || TH_TP64XX_BOARD)
	{
		printf("testing DSP-to-Host IRQ via HPI HINT...");
   

		// enable host IMBX#3 interrupts on BYTE#3 in order to enable extrenal PCIC IRQs
		TH_pcic_int_set_imbx_ie_on(3,3);

		// set DPRAM_MH_RQ IE bit in HIF IM register
		TH_set_im_rg(TP6X_IM_HPI_HINT_IE);

		// clear possible previous INTREG queue
		TH_clear_interrupt();
   
		TH_set_hpic_dspint();

		switch(err=TH_wait_interrupt(100))
		{
			case TH_INTERRUPT_NOT_DETECTED: // timed out
				err=TH_OK;
				printf("failed\n");
				strcpy(err_mess,"there is no reply from TORNADO via host INT");
				goto exit_error;

			case TH_INTERRUPT_DETECTED:// an interrupt occured -> process it
				if(TH_interrupt_registers(&regs))
				{
					err=TH_OK;
					// check interrupt source
					if(regs.id != TH_INT_ID_HPI_HINT)
					{
						printf("failed\n");
						strcpy(err_mess,"unexpected host INT source.");
						goto exit_error;
					}
				}
				else
				{
					printf("failed\n");
					strcpy(err_mess,"INT register queue is empty");
					goto exit_error;
				}

				break;

			default:	// this is error
				goto exit_error;
		}
	}

	data++;
	TH_clear_interrupt();
	printf("OK.\n");

	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;


	// successful demo termination
	// Reset the board and used resources.

	// disconnect TSDK host INT line
	if((err = TH_disconnect_interrupt())) goto exit_error;

	if(TH_TP6X_BOARD && TH_dpram_err_data())	
		goto dpram_error;

	// clear PCIC MBX flags and all possible IRQs
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// clear HPIC HINT for TP62/TP67/TP64xx
	if(TH_TP62_TP67_BOARD || TH_TP64XX_BOARD) 
		TH_clr_hpic_hint();		

	// clear DSP-to-Host request
	TH_clr_mh_rq();		

	// read/clear DSP-to-Host IRQ via DPRAM
	if(TH_TP6X_BOARD || (TH_TP64XX_BOARD && TH_board_dpram_len()>=TP64XX_DPRAM_LEN_256KX32))
		data = TH_dpram_mh_rq_data();

	// check for TSDK API error
	if(err=TH_err_flag()) goto exit_error;

	// now exit successfully
	err=TH_OK;

exit_error:
	_PCIC_CLR_ALL_MBX_STAT_FLAGS;
	_PCIC_CLR_ALL_IRQ;   
	TH_pcic_int_set_all_ie_off();

	// check for TSDK API errors
	if(err==TH_OK)
	   err=TH_err_flag();

	if(err!=TH_OK)
		printf("%s\n%s\n",TH_err_message(err),TH_err_flag_function_name());
	else
		printf(err_mess);

	// disconnect host INT line
	TH_disconnect_interrupt();

	// clear the CONTROL register (reset DSP, select HOST reset control, etc)
	TH_set_ctrl_rg(0);

	// clear the IMR register
	TH_set_im_rg(0);

	// clear the SMP register
	if(TH_TP6X_BOARD)
	{
		TH_set_smp0_rg(0);
		TH_set_smp1_rg(0);
		TH_clr_dpram_err();
	}

	// clear HPI error
	TH_clr_hpi_err();

	printf("\n");
}





